﻿using ProjectDumpTruck.Drawnings;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Security.Cryptography.X509Certificates;
using System.Text;
using System.Threading.Tasks;

namespace ProjectDumpTruck.CollectionGenericObject;

public class StorageCollection<T>
    where T : DrawningTruck
{
    /// <summary>
    /// Словарь (хранилище) с коллекциями
    /// </summary>
    readonly Dictionary<string, ICollectionGenericObject<T>> _storages;

    /// <summary>
    /// Возвращение списка названий коллекций
    /// </summary>
    public List<string> Keys => _storages.Keys.ToList();

    /// <summary>
    /// Ключевое слово, с которого должен начинаться файл
    /// </summary>
    private readonly string _collectionKey = "CollectionsStorage";
    /// <summary>
    /// Разделитель для записи ключа и значения элемента словаря
    /// </summary>
    private readonly string _separatorForKeyValue = "|";
    /// <summary>
    /// Разделитель для записей коллекции данных в файл
    /// </summary>
    private readonly string _separatorItems = ";";


    /// <summary>
    /// Конструктор
    /// </summary>
    public StorageCollection()
    {
        _storages = new Dictionary<string, ICollectionGenericObject<T>>();
    }

    /// <summary>
    /// Добавление коллекции в хранилище
    /// </summary>
    /// <param name="name">Название коллекции</param>
    /// <param name="collectionType">Тип коллекции</param>
    public void AddCollection(string name, CollectionType collectionType)
    {
        if (name == null || _storages.ContainsKey(name))
        {
            return;
        }

        if (collectionType == CollectionType.Massive)
        {
            _storages.Add(name, new MassiveGenericObjects<T>());
        }

        if (collectionType == CollectionType.List)
        {
            _storages.Add(name, new ListGenericObjects<T>());
        }
    }

    /// <summary>
    /// Удаление коллекции
    /// </summary>
    /// <param name="name">Название коллекции</param>
    public void DelCollection(string name)
    {
        if (name == null || !_storages.ContainsKey(name))
        {
            return;
        }

        _storages.Remove(name);
    }

    /// <summary>
    /// Доступ к коллекции
    /// </summary>
    /// <param name="name">Название коллекции</param>
    /// <returns></returns>
    public ICollectionGenericObject<T>? this[string name]
    {
        get
        {
            if (_storages.ContainsKey(name))
            {
                return _storages[name];
            }

            return null;
        }
    }

    public bool SaveData(string filename)
    {
        if (File.Exists(filename)) File.Delete(filename);

        if (_storages.Count == 0) return false;

        using (StreamWriter sw = new StreamWriter(filename))
        {
            sw.Write(_collectionKey);

            foreach (KeyValuePair<string, ICollectionGenericObject<T>> value in _storages)
            {
                sw.Write(Environment.NewLine);
                // не сохраняем пустые коллекции
                if (value.Value.Count == 0) continue;

                sw.Write(value.Key);
                sw.Write(_separatorForKeyValue);
                sw.Write(value.Value.GetCollectionType);
                sw.Write(_separatorForKeyValue);
                sw.Write(value.Value.MaxCount);
                sw.Write(_separatorForKeyValue);

                foreach (T? item in value.Value.GetItems())
                {
                    string data = item?.GetDataForSave() ?? string.Empty;

                    if (string.IsNullOrEmpty(data)) continue;

                    sw.Write(data);
                    sw.Write(_separatorItems);
                }
            }
        }

        return true;
    }

    /// <summary>
    /// Загрузка информации по автомобилям в хранилище из файла
    /// </summary>
    /// <param name="filename">Путь и имя файла</param>
    /// <returns>true - загрузка прошла успешно, false - ошибка при загрузке данных</returns>
    public bool LoadData(string filename)
    {
        if (!File.Exists(filename)) return false;

        using (StreamReader sr = new(filename))
        {
            string line = sr.ReadLine();

            if (line == null || line.Length == 0) return false;

            if (!line.Equals(_collectionKey))
            {
                //если нет такой записи, то это не те данные
                return false;
            }
            _storages.Clear();

            while ((line = sr.ReadLine()) != null)
            {
                string[] record = line.Split(_separatorForKeyValue, StringSplitOptions.RemoveEmptyEntries);
                if (record.Length != 4)
                {
                    continue;
                }

                CollectionType collectionType = (CollectionType)Enum.Parse(typeof(CollectionType), record[1]);
                ICollectionGenericObject<T>? collection = StorageCollection<T>.CreateCollection(collectionType);
                if (collection == null)
                {
                    return false;
                }

                collection.MaxCount = Convert.ToInt32(record[2]);
                string[] set = record[3].Split(_separatorItems, StringSplitOptions.RemoveEmptyEntries);
                foreach (string elem in set)
                {
                    if (elem?.CreateDrawningTruck() is T truck)
                    {
                        if (collection.Insert(truck) == -1)
                        {
                            return false;
                        }
                    }
                }
                _storages.Add(record[0], collection);
            }
        }

        return true;
    }

    /// <summary>
    /// Создание коллекции по типу
    /// </summary>
    /// <param name="collectionType"></param>
    /// <returns></returns>
    private static ICollectionGenericObject<T>? CreateCollection(CollectionType collectionType)
    {
        return collectionType switch
        {
            CollectionType.Massive => new MassiveGenericObjects<T>(),
            CollectionType.List => new ListGenericObjects<T>(),
            _ => null,
        };
    }

}

